@@ -1,9 +1,9 @@
-## dist.topo.R (2015-04-24)
+## dist.topo.R (2017-07-28)
 
 ##      Topological Distances, Tree Bipartitions,
 ##   Consensus Trees, and Bootstrapping Phylogenies
 
-## Copyright 2005-2015 Emmanuel Paradis
+## Copyright 2005-2017 Emmanuel Paradis, 2016-2017 Klaus Schliep
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -17,89 +17,106 @@
 
 dist.topo <- function(x, y = NULL, method = "PH85")
 {
-    if (!is.null(y)) {
-        res <- .dist.topo(x, y, method = method)
-    } else {
-        n <- length(x) # number of trees
-        res <- numeric(n * (n - 1) /2)
-        nms <- names(x)
-        if (is.null(nms)) nms <- paste0("tree", 1:n)
+    method <- match.arg(method, c("PH85", "score"))
+    if (!is.null(y)) x <- c(x, y)
+    testroot <- any(is.rooted(x))
+    n <- length(x) # number of trees
+    res <- numeric(n * (n - 1) /2)
+    nms <- names(x)
+    if (is.null(nms)) nms <- paste0("tree", 1:n)
+
+    if (method == "PH85") {
+        if (testroot)
+            warning("Some trees were rooted: topological distances may be spurious.")
+
+        x <- .compressTipLabel(x)
+        ntip <- length(attr(x, "TipLabel"))
+        nnode <- sapply(x, Nnode)
+
+        foo <- function(phy, ntip) {
+            phy <- reorder(phy, "postorder")
+            ans <- ONEwise(bipartition2(phy$edge, ntip))
+            sapply(ans, paste, collapse = "\r")
+        }
+
+        x <- lapply(x, foo, ntip = ntip)
+
+        k <- 0L
+        for (i in 1:(n - 1)) {
+            y <- x[[i]]
+            m1 <- nnode[i]
+            for (j in (i + 1):n) {
+                z <- x[[j]]
+                k <- k + 1L
+                res[k] <- m1 + nnode[j] - 2 * sum(z %in% y)
+            }
+        }
+    } else { # method == "score"
         k <- 0L
         for (i in 1:(n - 1)) {
             for (j in (i + 1):n) {
                 k <- k + 1L
-                res[k] <- .dist.topo(x[[i]], x[[j]], method = method)
+                ## still very slow......
+                res[k] <- .dist.topo.score(x[[i]], x[[j]])
             }
         }
-        attr(res, "Size") <- n
-        attr(res, "Labels") <- nms
-        attr(res, "Diag") <- attr(res, "Upper") <- FALSE
-        attr(res, "method") <- method
-        class(res) <- "dist"
     }
+
+    attr(res, "Size") <- n
+    attr(res, "Labels") <- nms
+    attr(res, "Diag") <- attr(res, "Upper") <- FALSE
+    attr(res, "method") <- method
+    class(res) <- "dist"
     res
 }
 
-.dist.topo <- function(x, y, method = "PH85")
+.dist.topo.score <- function(x, y)
 {
-    if (method == "score" && (is.null(x$edge.length) || is.null(y$edge.length)))
+    if (is.null(x$edge.length) || is.null(y$edge.length))
         stop("trees must have branch lengths for branch score distance.")
     nx <- length(x$tip.label)
-    x <- unroot(x)
-    y <- unroot(y)
-    bp1 <- .Call(bipartition, x$edge, nx, x$Nnode)
+    x <- reorder.phylo(unroot(x), "postorder")
+    y <- reorder.phylo(unroot(y), "postorder")
+    ##bp1 <- .Call(bipartition, x$edge, nx, x$Nnode)
+    bp1 <- bipartition2(x$edge, nx)
     bp1 <- lapply(bp1, function(xx) sort(x$tip.label[xx]))
     ny <- length(y$tip.label) # fix by Otto Cordero
     ## fix by Tim Wallstrom:
-    bp2.tmp <- .Call(bipartition, y$edge, ny, y$Nnode)
+    bp2.tmp <- bipartition2(y$edge, ny)
+    ##bp2.tmp <- .Call(bipartition, y$edge, ny, y$Nnode)
     bp2 <- lapply(bp2.tmp, function(xx) sort(y$tip.label[xx]))
     bp2.comp <- lapply(bp2.tmp, function(xx) setdiff(1:ny, xx))
     bp2.comp <- lapply(bp2.comp, function(xx) sort(y$tip.label[xx]))
     ## End
     q1 <- length(bp1)
     q2 <- length(bp2)
-    if (method == "PH85") {
-        p <- 0
-        for (i in 1:q1) {
-            for (j in 1:q2) {
-                if (identical(bp1[[i]], bp2[[j]]) | identical(bp1[[i]], bp2.comp[[j]])) {
-                    p <- p + 1
-                    break
-                }
+
+    dT <- 0
+    found1 <- FALSE
+    found2 <- logical(q2)
+    found2[1] <- TRUE
+    for (i in 2:q1) {
+        for (j in 2:q2) {
+            if (identical(bp1[[i]], bp2[[j]]) | identical(bp1[[i]], bp2.comp[[j]])) {
+                dT <- dT + (x$edge.length[which(x$edge[, 2] == nx + i)] -
+                            y$edge.length[which(y$edge[, 2] == ny + j)])^2
+                found1 <- found2[j] <- TRUE
+                break
             }
         }
-        dT <- q1 + q2 - 2 * p # same than:
-        ##dT <- if (q1 == q2) 2*(q1 - p) else 2*(min(q1, q2) - p) + abs(q1 - q2)
+        if (found1) found1 <- FALSE
+        else dT <- dT + (x$edge.length[which(x$edge[, 2] == nx + i)])^2
     }
-    if (method == "score") {
-        dT <- 0
-        found1 <- FALSE
-        found2 <- logical(q2)
-        found2[1] <- TRUE
-        for (i in 2:q1) {
-            for (j in 2:q2) {
-                if (identical(bp1[[i]], bp2[[j]]) | identical(bp1[[i]], bp2.comp[[j]])) {
-                    dT <- dT + (x$edge.length[which(x$edge[, 2] == nx + i)] -
-                                y$edge.length[which(y$edge[, 2] == ny + j)])^2
-                    found1 <- found2[j] <- TRUE
-                    break
-                }
-            }
-            if (found1) found1 <- FALSE
-            else dT <- dT + (x$edge.length[which(x$edge[, 2] == nx + i)])^2
-        }
-        if (!all(found2))
-            dT <- dT + sum((y$edge.length[y$edge[, 2] %in% (ny + which(!found2))])^2)
-        dT <- sqrt(dT)
-    }
-    dT
+    if (!all(found2))
+        dT <- dT + sum((y$edge.length[y$edge[, 2] %in% (ny + which(!found2))])^2)
+    sqrt(dT)
 }
 
-.compressTipLabel <- function(x)
+.compressTipLabel <- function(x, ref = NULL)
 {
     ## 'x' is a list of objects of class "phylo" possibly with no class
     if (!is.null(attr(x, "TipLabel"))) return(x)
-    ref <- x[[1]]$tip.label
+    if (is.null(ref)) ref <- x[[1]]$tip.label
     n <- length(ref)
     if (length(unique(ref)) != n)
         stop("some tip labels are duplicated in tree no. 1")
@@ -135,11 +152,12 @@ prop.part <- function(..., check.labels = TRUE)
     class(obj) <- NULL # fix by Klaus Schliep (2014-03-06)
     for (i in 1:ntree) storage.mode(obj[[i]]$Nnode) <- "integer"
     class(obj) <- "multiPhylo"
-    obj <- .uncompressTipLabel(obj) # fix a bug (2010-11-18)
-    clades <- .Call(prop_part, obj, ntree, TRUE)
-    attr(clades, "number") <- attr(clades, "number")[1:length(clades)]
+    obj <- reorder(obj, "postorder")
+# the following line should not be necessary any more
+#    obj <- .uncompressTipLabel(obj) # fix a bug (2010-11-18)
+    nTips <- length(obj[[1]]$tip.label)
+    clades <- prop_part2(obj, nTips)
     attr(clades, "labels") <- obj[[1]]$tip.label
-    class(clades) <- "prop.part"
     clades
 }
 
@@ -180,75 +198,92 @@ plot.prop.part <- function(x, barcol = "blue", leftmar = 4, col = "red", ...)
     mtext(attr(x, "labels"), side = 2, at = 1:n, las = 1)
 }
 
+### by Klaus (2016-03-23):
 prop.clades <- function(phy, ..., part = NULL, rooted = FALSE)
 {
     if (is.null(part)) {
         obj <- .getTreesFromDotdotdot(...)
+        ## avoid double counting of edges if trees are rooted
+        if (!rooted) obj <- lapply(obj, unroot)
         part <- prop.part(obj, check.labels = TRUE)
     }
-
-    ## until ape 3.0-7 it was assumed implicitly that the labels in phy
-    ## are in the same order than in 'part' (bug report by Rupert Collins)
-    if (!identical(phy$tip.label, attr(part, "labels"))) {
-        i <- match(phy$tip.label, attr(part, "labels"))
+    LABS <- attr(part, "labels")
+    if (!identical(phy$tip.label, LABS)) {
+        i <- match(phy$tip.label, LABS)
         j <- match(seq_len(Ntip(phy)), phy$edge[, 2])
         phy$edge[j, 2] <- i
-        phy$tip.label <- attr(part, "labels")
+        phy$tip.label <- LABS
     }
     bp <- prop.part(phy)
     if (!rooted) {
-        bp <- postprocess.prop.part(bp)
-        part <- postprocess.prop.part(part) # fix by Klaus Schliep
-        ## actually the above line in not needed if called from boot.phylo()
-    }
-
-    n <- numeric(phy$Nnode)
-    for (i in seq_along(bp)) {
-        for (j in seq_along(part)) {
-            ## we rely on the fact the values returned by prop.part are
-            ## sorted and without attributes, so identical can be used:
-            if (identical(bp[[i]], part[[j]])) {
-                n[i] <- attr(part, "number")[j]
-                done <-  TRUE
-                break
-            }
-        }
+        ## avoid messing up the order and length if phy is rooted in some cases
+        bp <- ONEwise(bp)
+        part <- postprocess.prop.part(part)
     }
+    pos <- match(bp, part)
+    tmp <- which(!is.na(pos))
+    n <- rep(NA_real_, phy$Nnode)
+    n[tmp] <- attr(part, "number")[pos[tmp]]
     n
 }
 
-boot.phylo <- function(phy, x, FUN, B = 100, block = 1,
-                       trees = FALSE, quiet = FALSE, rooted = FALSE)
+boot.phylo <-
+    function(phy, x, FUN, B = 100, block = 1,
+             trees = FALSE, quiet = FALSE,
+             rooted = is.rooted(phy), jumble = TRUE,
+             mc.cores = 1)
 {
     if (is.null(dim(x)) || length(dim(x)) != 2)
         stop("the data 'x' must have two dimensions (e.g., a matrix or a data frame)")
 
+    if (anyDuplicated(rownames(x)))
+        stop("some labels are duplicated in the data: you won't be able to analyse tree bipartitions")
+
     boot.tree <- vector("list", B)
-
-    if (!quiet) # suggestion by Alastair Potts
-        progbar <- txtProgressBar(style = 3)
-
     y <- nc <- ncol(x)
+    nr <- nrow(x)
 
     if (block > 1) {
         a <- seq(1, nc - 1, block)
         b <- seq(block, nc, block)
         y <- mapply(":", a, b, SIMPLIFY = FALSE)
+        getBootstrapIndices <- function() unlist(sample(y, replace = TRUE))
+    } else getBootstrapIndices <- function() sample.int(y, replace = TRUE)
+
+    if (!quiet) {
+        prefix <- "\rRunning bootstraps:      "
+        suffix <- paste("/", B)
+        updateProgress <- function(i) cat(prefix, i, suffix)
     }
 
-    for (i in 1:B) {
-        boot.samp <- unlist(sample(y, replace = TRUE))
-        boot.tree[[i]] <- FUN(x[, boot.samp])
-        if (!quiet) setTxtProgressBar(progbar, i/B)
+    if (mc.cores == 1) {
+        for (i in 1:B) {
+            boot.samp <- x[, getBootstrapIndices()]
+            if (jumble) boot.samp <- boot.samp[sample.int(nr), ]
+            boot.tree[[i]] <- FUN(boot.samp)
+            if (!quiet && !(i %% 100)) updateProgress(i)
+        }
+    } else {
+        if (!quiet) cat("Running parallel bootstraps...")
+        foo <- function(i) {
+            boot.samp <- x[, getBootstrapIndices()]
+            if (jumble) boot.samp <- boot.samp[sample.int(nr), ]
+            FUN(boot.samp)
+        }
+        boot.tree <- mclapply(1:B, foo, mc.cores = mc.cores)
+        if (!quiet) cat(" done.")
     }
-    if (!quiet) close(progbar)
 
-    ## for (i in 1:B) storage.mode(boot.tree[[i]]$Nnode) <- "integer"
-    ## storage.mode(phy$Nnode) <- "integer"
+    if (!quiet) cat("\nCalculating bootstrap values...")
 
-    if (!quiet) cat("Calculating bootstrap values...")
+    ## sort labels after mixed them up
+    if (jumble) {
+        boot.tree <- .compressTipLabel(boot.tree, ref = phy$tip.label)
+        boot.tree <- .uncompressTipLabel(boot.tree)
+        boot.tree <- unclass(boot.tree) # otherwise countBipartitions crashes
+    }
 
-     if (rooted) {
+    if (rooted) {
         pp <- prop.part(boot.tree)
         ans <- prop.clades(phy, part = pp, rooted = rooted)
     } else {
@@ -258,11 +293,12 @@ boot.phylo <- function(phy, x, FUN, B = 100, block = 1,
         ans <- c(B, ans[order(phy$edge[ints, 2])])
     }
 
+    if (!quiet) cat(" done.\n")
+
     if (trees) {
         class(boot.tree) <- "multiPhylo"
         ans <- list(BP = ans, trees = boot.tree)
     }
-    if (!quiet) cat(" done.\n")
     ans
 }
 
@@ -272,39 +308,27 @@ boot.phylo <- function(phy, x, FUN, B = 100, block = 1,
 ### split though they are different clades. The aggregation is done
 ### arbitrarily. The call to ONEwise() insures that all splits include
 ### the first tip.
+### (rewritten by Klaus)
 postprocess.prop.part <- function(x)
 {
-    n <- length(x[[1]])
-    N <- length(x)
     w <- attr(x, "number")
+    labels <- attr(x, "labels")
+
+    x <- ONEwise(x)
+    drop <- duplicated(x)
 
-    drop <- logical(N)
-    V <- numeric(n)
-    for (i in 2:(N - 1)) {
-        if (drop[i]) next
-        A <- x[[i]]
-        for (j in (i + 1):N) {
-            if (drop[j]) next
-            B <- x[[j]]
-            if (length(A) + length(B) != n) next
-            V[] <- 0L
-            V[A] <- 1L
-            V[B] <- 1L
-            if (all(V == 1L)) {
-                drop[j] <- TRUE
-                w[i] <- w[i] + w[j]
-            }
-        }
-    }
     if (any(drop)) {
-        labels <- attr(x, "labels")
+        ind1 <- match(x[drop], x)
+        ind2 <- which(drop)
+        for (i in seq_along(ind2))
+            w[ind1[i]] <- w[ind1[i]] + w[ind2[i]]
         x <- x[!drop]
         w <- w[!drop]
-        attr(x, "number") <- w
-        attr(x, "labels") <- labels
-        class(x) <- "prop.part"
     }
-    ONEwise(x)
+    attr(x, "number") <- w
+    attr(x, "labels") <- labels
+    class(x) <- "prop.part"
+    x
 }
 
 ### This function changes an object of class "prop.part" so that they
@@ -312,8 +336,7 @@ postprocess.prop.part <- function(x)
 ### changed to 1:2.
 ONEwise <- function(x)
 {
-    n <- length(x[[1L]])
-    v <- 1:n
+    v <- seq_along(x[[1L]])
     for (i in 2:length(x)) {
         y <- x[[i]]
         if (y[1] != 1) x[[i]] <- v[-y]
@@ -363,8 +386,7 @@ consensus <- function(..., p = 1, check.labels = TRUE)
     if (p == 0.5) p <- 0.5000001 # avoid incompatible splits
     pp <- pp[attr(pp, "number") >= p * ntree]
     ## Get the order of the remaining partitions by decreasing size:
-    ind <- sort(unlist(lapply(pp, length)), decreasing = TRUE,
-                index.return = TRUE)$ix
+    ind <- order(lengths(pp), decreasing = TRUE)
     pp <- pp[ind]
     n <- length(labels)
     m <- length(pp)
